
import { write_text } from '../fs';
import { BuildState } from './state';
import { map_input_file_to_output_file } from './helpers';
import { parseICS, CalendarComponent } from 'ical';
import create_calendar, { ICalEventData, ICalCalendarData } from 'ical-generator';

export type { ICalEventData, ICalCalendarData, ICalAttendeeData, ICalAttendeeStatus } from 'ical-generator';

export interface EventEntry {
	url: string;
	in_file: string;
	title?: string;
	description?: string;
	author_name?: string;
	author_email?: string;
	start_time?: string;
	end_time?: string;
	time_zone?: `${string}/${string}`;
}

export async function write_events_and_calendars_if_needed(state: BuildState) {
	if (state.conf.events) {
		for (const entry of state.events) {
			const event = icalendar_event(state, entry);
			const cal_data: ICalCalendarData = {
				prodId: {
					company: 'jbrumond.me',
					product: 'docs2website',
					language: 'EN',
				},
				// ...
			};

			const calendar = create_icalendar(cal_data, event);
			const out_file = await map_input_file_to_output_file(state, entry.in_file, [ '.html', '.md', '.markdown' ], '.isc');
			await write_text(out_file, calendar);
		}
	}

	if (state.conf.calendars) {
		for (let index = 0; index < state.conf.calendars.length; index++) {
			const cal_conf = state.conf.calendars[index];
			const cal_entries = state.calendars[index];

			for (const entry of cal_entries) {
				const event = icalendar_event(state, entry);
				const cal_data: ICalCalendarData = {
					name: cal_conf.title,
					prodId: {
						company: 'jbrumond.me',
						product: 'docs2website',
						language: 'EN',
					},
					// ...
				};

				const calendar = create_icalendar(cal_data, event);
				await write_text(cal_conf.out_file, calendar);
			}
		}
	}
}

export function parse_icalendar(contents: string) {
	const parsed = parseICS(contents);
	const calendar: CalendarComponent[] = [ ];

	for (const data of Object.values(parsed)) {
		calendar.push(data);
	}

	return calendar;
}

export function create_icalendar(cal: ICalCalendarData, events: ICalEventData | ICalEventData[]) {
	const calendar = create_calendar(cal);

	if (Array.isArray(events)) {
		for (const event of events) {
			calendar.createEvent(event);
		}
	}

	else {
		calendar.createEvent(events);
	}

	return calendar.toString();
}

export function icalendar_event(state: BuildState, entry: EventEntry) : ICalEventData {
	const in_file = entry.in_file.slice(state.conf.input.root.length);
	const metadata = state.new_metadata.files[in_file];

	return {
		id: entry.url,
		summary: entry.title,
		description: entry.description || void 0,
		start: entry.start_time,
		end: entry.end_time,
		url: entry.url,
		timezone: entry.time_zone,
		created: metadata.first_seen_time,
		lastModified: metadata.last_updated_time,
		organizer: {
			name: entry.author_name || 'Unknown',
			email: entry.author_email,
		},
		// attendees: post.mentions.flatMap((mention) : ICalAttendeeData | ICalAttendeeData[] => {
		// 	if (mention.is_rsvp && mention.is_reply_to_this) {
		// 		const ext = mention.external as ExternalEntry;
		// 		const status = ext.rsvp_type === 'yes'
		// 			? 'ACCEPTED' as const
		// 			: ext.rsvp_type === 'no'
		// 				? 'DECLINED' as const
		// 				: ext.rsvp_type === 'maybe'
		// 					? 'TENTATIVE' as const
		// 					: 'NEEDS-ACTION' as const;

		// 		return {
		// 			name: mention.author_name,
		// 			rsvp: ext.rsvp_type === 'yes',
		// 			status: status as ICalAttendeeStatus
		// 		};
		// 	}

		// 	return [ ];
		// }),
	};
}
